@@ -1,8 +1,6 @@
 module Language.Haskell.GhcMod.Browse (
-    browseModule
-  , browse
-  , browseAll)
-  where
+    browse
+  ) where
 
 import Control.Applicative ((<$>))
 import Control.Exception (SomeException(..))
@@ -11,11 +9,13 @@ import Data.List (sort)
 import Data.Maybe (catMaybes)
 import Exception (ghandle)
 import FastString (mkFastString)
-import GHC (Ghc, GhcException(CmdLineError), ModuleInfo, Name, TyThing, DynFlags, Type, TyCon, Module)
+import GHC (GhcException(CmdLineError), ModuleInfo, Name, TyThing, DynFlags, Type, TyCon)
 import qualified GHC as G
-import Language.Haskell.GhcMod.Doc (showPage, showOneLine, styleUnqualified)
-import Language.Haskell.GhcMod.GHCApi
+import Language.Haskell.GhcMod.Convert
+import Language.Haskell.GhcMod.Doc (showPage, styleUnqualified)
 import Language.Haskell.GhcMod.Gap
+import Language.Haskell.GhcMod.Monad (GhcModT, options)
+import Language.Haskell.GhcMod.Target (setTargetFiles)
 import Language.Haskell.GhcMod.Types
 import Name (getOccString)
 import Outputable (ppr, Outputable)
@@ -27,28 +27,16 @@ import Type (dropForAlls, splitFunTy_maybe, mkFunTy, isPredTy)
 -- | Getting functions, classes, etc from a module.
 --   If 'detailed' is 'True', their types are also obtained.
 --   If 'operators' is 'True', operators are also returned.
-browseModule :: Options
-             -> Cradle
-             -> ModuleString -- ^ A module name. (e.g. \"Data.List\")
-             -> IO String
-browseModule opt cradle pkgmdl = withGHC' $ do
-    initializeFlagsWithCradle opt cradle
-    browse opt pkgmdl
-
--- | Getting functions, classes, etc from a module.
---   If 'detailed' is 'True', their types are also obtained.
---   If 'operators' is 'True', operators are also returned.
-browse :: Options
-       -> ModuleString -- ^ A module name. (e.g. \"Data.List\")
-       -> Ghc String
-browse opt pkgmdl = do
-    convert opt . sort <$> (getModule >>= listExports)
+browse :: IOish m
+       => ModuleString -- ^ A module name. (e.g. \"Data.List\")
+       -> GhcModT m String
+browse pkgmdl = convert' . sort =<< (listExports =<< getModule)
   where
     (mpkg,mdl) = splitPkgMdl pkgmdl
     mdlname = G.mkModuleName mdl
     mpkgid = mkFastString <$> mpkg
     listExports Nothing       = return []
-    listExports (Just mdinfo) = processExports opt mdinfo
+    listExports (Just mdinfo) = processExports mdinfo
     -- findModule works only for package modules, moreover,
     -- you cannot load a package module. On the other hand,
     -- to browse a local module you need to load it first.
@@ -73,19 +61,22 @@ splitPkgMdl pkgmdl = case break (==':') pkgmdl of
     (mdl,"")    -> (Nothing,mdl)
     (pkg,_:mdl) -> (Just pkg,mdl)
 
-processExports :: Options -> ModuleInfo -> Ghc [String]
-processExports opt minfo = mapM (showExport opt minfo) $ removeOps $ G.modInfoExports minfo
-  where
+processExports :: IOish m => ModuleInfo -> GhcModT m [String]
+processExports minfo = do
+  opt <- options
+  let
     removeOps
       | operators opt = id
       | otherwise = filter (isAlpha . head . getOccString)
+  mapM (showExport opt minfo) $ removeOps $ G.modInfoExports minfo
 
-showExport :: Options -> ModuleInfo -> Name -> Ghc String
+showExport :: IOish m => Options -> ModuleInfo -> Name -> GhcModT m String
 showExport opt minfo e = do
   mtype' <- mtype
   return $ concat $ catMaybes [mqualified, Just $ formatOp $ getOccString e, mtype']
   where
     mqualified = (G.moduleNameString (G.moduleName $ G.nameModule e) ++ ".") `justIf` qualified opt
+    mtype :: IOish m => GhcModT m (Maybe String)
     mtype
       | detailed opt = do
         tyInfo <- G.modInfoLookupName minfo e
@@ -100,7 +91,7 @@ showExport opt minfo e = do
       | isAlpha n = nm
       | otherwise = "(" ++ nm ++ ")"
     formatOp "" = error "formatOp"
-    inOtherModule :: Name -> Ghc (Maybe TyThing)
+    inOtherModule :: IOish m => Name -> GhcModT m (Maybe TyThing)
     inOtherModule nm = G.getModuleInfo (G.nameModule nm) >> G.lookupGlobalName nm
     justIf :: a -> Bool -> Maybe a
     justIf x True = Just x
@@ -143,20 +134,3 @@ removeForAlls' ty (Just (pre, ftype))
 
 showOutputable :: Outputable a => DynFlags -> a -> String
 showOutputable dflag = unwords . lines . showPage dflag styleUnqualified . ppr
-
-----------------------------------------------------------------
-
--- | Browsing all functions in all system/user modules.
-browseAll :: DynFlags -> Ghc [(String,String)]
-browseAll dflag = do
-    ms <- G.packageDbModules True
-    is <- mapM G.getModuleInfo ms
-    return $ concatMap (toNameModule dflag) (zip ms is)
-
-toNameModule :: DynFlags -> (Module, Maybe ModuleInfo) -> [(String,String)]
-toNameModule _     (_,Nothing)  = []
-toNameModule dflag (m,Just inf) = map (\name -> (toStr name, mdl)) names
-  where
-    mdl = G.moduleNameString (G.moduleName m)
-    names = G.modInfoExports inf
-    toStr = showOneLine dflag styleUnqualified . ppr
