@@ -7,18 +7,22 @@ module Language.Haskell.GhcMod.CabalApi (
   , cabalDependPackages
   , cabalSourceDirs
   , cabalAllTargets
+  , cabalConfigDependencies
   ) where
 
-import Language.Haskell.GhcMod.Types
+import Language.Haskell.GhcMod.CabalConfig
+import Language.Haskell.GhcMod.Error
+import Language.Haskell.GhcMod.Gap (benchmarkBuildInfo, benchmarkTargets,
+                                    toModuleString)
 import Language.Haskell.GhcMod.GhcPkg
+import Language.Haskell.GhcMod.Types
 
+import MonadUtils (liftIO)
 import Control.Applicative ((<$>))
-import Control.Exception (throwIO)
+import qualified Control.Exception as E
 import Control.Monad (filterM)
-import CoreMonad (liftIO)
-import Data.Maybe (maybeToList, catMaybes)
+import Data.Maybe (maybeToList)
 import Data.Set (fromList, toList)
-import Distribution.ModuleName (ModuleName,toFilePath)
 import Distribution.Package (Dependency(Dependency)
                            , PackageName(PackageName))
 import qualified Distribution.Package as C
@@ -27,56 +31,31 @@ import qualified Distribution.PackageDescription as P
 import Distribution.PackageDescription.Configuration (finalizePackageDescription)
 import Distribution.PackageDescription.Parse (readPackageDescription)
 import Distribution.Simple.Compiler (CompilerId(..), CompilerFlavor(..))
-import Distribution.Simple.Program (ghcProgram)
+import Distribution.Simple.Program as C (ghcProgram)
 import Distribution.Simple.Program.Types (programName, programFindVersion)
 import Distribution.System (buildPlatform)
 import Distribution.Text (display)
 import Distribution.Verbosity (silent)
 import Distribution.Version (Version)
 import System.Directory (doesFileExist)
-import System.FilePath (dropExtension, takeFileName, (</>))
-
+import System.FilePath ((</>))
 ----------------------------------------------------------------
 
 -- | Getting necessary 'CompilerOptions' from three information sources.
-getCompilerOptions :: [GHCOption]
+getCompilerOptions :: (IOish m, MonadError GhcModError m)
+                   => [GHCOption]
                    -> Cradle
                    -> PackageDescription
-                   -> IO CompilerOptions
+                   -> m CompilerOptions
 getCompilerOptions ghcopts cradle pkgDesc = do
-    gopts <- getGHCOptions ghcopts cradle rdir $ head buildInfos
-    dbPkgs <- ghcPkgListEx (cradlePkgDbStack cradle)
-    return $ CompilerOptions gopts idirs (depPkgs dbPkgs)
+    gopts <- liftIO $ getGHCOptions ghcopts cradle rdir $ head buildInfos
+    depPkgs <- cabalConfigDependencies cradle (C.packageId pkgDesc)
+    return $ CompilerOptions gopts idirs depPkgs
   where
     wdir       = cradleCurrentDir cradle
     rdir       = cradleRootDir    cradle
-    Just cfile = cradleCabalFile  cradle
-    thisPkg    = dropExtension $ takeFileName cfile
     buildInfos = cabalAllBuildInfo pkgDesc
     idirs      = includeDirectories rdir wdir $ cabalSourceDirs buildInfos
-    depPkgs ps = attachPackageIds ps
-                   $ removeThem (problematicPackages ++ [thisPkg])
-                   $ cabalDependPackages buildInfos
-
-----------------------------------------------------------------
--- Dependent packages
-
-removeThem :: [PackageBaseName] -> [PackageBaseName] -> [PackageBaseName]
-removeThem badpkgs = filter (`notElem` badpkgs)
-
-problematicPackages :: [PackageBaseName]
-problematicPackages = [
-    "base-compat" -- providing "Prelude"
-  ]
-
-attachPackageIds :: [Package] -> [PackageBaseName] -> [Package]
-attachPackageIds pkgs = catMaybes . fmap (`lookup3` pkgs)
-
-lookup3 :: Eq a => a -> [(a,b,c)] -> Maybe (a,b,c)
-lookup3 _ [] = Nothing
-lookup3 k (t@(a,_,_):ls)
-    | k == a = Just t
-    | otherwise = lookup3 k ls
 
 ----------------------------------------------------------------
 -- Include directories for modules
@@ -93,19 +72,23 @@ includeDirectories cdir wdir dirs = uniqueAndSort (extdirs ++ [cdir,wdir])
 
 ----------------------------------------------------------------
 
--- | Parsing a cabal file and returns 'PackageDescription'.
---   'IOException' is thrown if parsing fails.
-parseCabalFile :: FilePath -> IO PackageDescription
-parseCabalFile file = do
-    cid <- getGHCId
-    epgd <- readPackageDescription silent file
-    case toPkgDesc cid epgd of
-        Left deps    -> throwIO $ userError $ show deps ++ " are not installed"
+-- | Parse a cabal file and return a 'PackageDescription'.
+parseCabalFile :: (IOish m, MonadError GhcModError m)
+               => Cradle
+               -> FilePath
+               -> m PackageDescription
+parseCabalFile cradle file = do
+    cid <- liftIO getGHCId
+    epgd <- liftIO $ readPackageDescription silent file
+    flags <- cabalConfigFlags cradle
+    case toPkgDesc cid flags epgd of
+        Left deps    -> fail $ show deps ++ " are not installed"
         Right (pd,_) -> if nullPkg pd
-                        then throwIO $ userError $ file ++ " is broken"
+                        then fail $ file ++ " is broken"
                         else return pd
   where
-    toPkgDesc cid = finalizePackageDescription [] (const True) buildPlatform cid []
+    toPkgDesc cid flags =
+        finalizePackageDescription flags (const True) buildPlatform cid []
     nullPkg pd = name == ""
       where
         PackageName name = C.pkgName (P.package pd)
@@ -143,11 +126,7 @@ cabalAllBuildInfo pd = libBI ++ execBI ++ testBI ++ benchBI
     libBI   = map P.libBuildInfo       $ maybeToList $ P.library pd
     execBI  = map P.buildInfo          $ P.executables pd
     testBI  = map P.testBuildInfo      $ P.testSuites pd
-#if __GLASGOW_HASKELL__ >= 704
-    benchBI = map P.benchmarkBuildInfo $ P.benchmarks pd
-#else
-    benchBI = []
-#endif
+    benchBI = benchmarkBuildInfo pd
 
 ----------------------------------------------------------------
 
@@ -176,9 +155,10 @@ getGHCId = CompilerId GHC <$> getGHC
 
 getGHC :: IO Version
 getGHC = do
-    mv <- programFindVersion ghcProgram silent (programName ghcProgram)
+    mv <- programFindVersion C.ghcProgram silent (programName C.ghcProgram)
     case mv of
-        Nothing -> throwIO $ userError "ghc not found"
+      -- TODO: MonadError it up
+        Nothing -> E.throwIO $ userError "ghc not found"
         Just v  -> return v
 
 ----------------------------------------------------------------
@@ -196,16 +176,7 @@ cabalAllTargets pd = do
             Just l -> P.libModules l
 
     libTargets = map toModuleString lib
-#if __GLASGOW_HASKELL__ >= 704
-    benchTargets = map toModuleString $ concatMap P.benchmarkModules $ P.benchmarks  pd
-#else
-    benchTargets = []
-#endif
-    toModuleString :: ModuleName -> String
-    toModuleString mn = fromFilePath $ toFilePath mn
-
-    fromFilePath :: FilePath -> String
-    fromFilePath fp = map (\c -> if c=='/' then '.' else c) fp
+    benchTargets = benchmarkTargets pd
 
     getTestTarget :: TestSuite -> IO [String]
     getTestTarget ts =
