@@ -28,7 +28,7 @@
 	       (< emacs-minor-version minor)))
       (error "ghc-mod requires at least Emacs %d.%d" major minor)))
 
-(defconst ghc-version "4.1.1")
+(defconst ghc-version "5.2.1.1")
 
 ;; (eval-when-compile
 ;;  (require 'haskell-mode))
@@ -40,6 +40,7 @@
 (require 'ghc-command)
 (require 'ghc-ins-mod)
 (require 'ghc-indent)
+(require 'ghc-rewrite)
 (require 'dabbrev)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -63,7 +64,6 @@
 (defvar ghc-sort-key        "\es")
 (defvar ghc-type-key        "\C-c\C-t")
 (defvar ghc-info-key        "\C-c\C-i")
-(defvar ghc-check-key       "\C-x\C-s")
 (defvar ghc-toggle-key      "\C-c\C-c")
 (defvar ghc-jump-key        "\C-c\C-j")
 (defvar ghc-module-key      "\C-c\C-m")
@@ -72,6 +72,11 @@
 (defvar ghc-hoogle-key      (format "\C-c%c" (ghc-find-C-h)))
 (defvar ghc-shallower-key   "\C-c<")
 (defvar ghc-deeper-key      "\C-c>")
+;(defvar ghc-case-split-key  "\C-c\C-s")
+(defvar ghc-refine-key      "\C-c\C-f")
+(defvar ghc-auto-key        "\C-c\C-a")
+(defvar ghc-prev-hole-key   "\C-c\ep")
+(defvar ghc-next-hole-key   "\C-c\en")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -94,9 +99,8 @@
     (define-key haskell-mode-map ghc-previous-key    'ghc-goto-prev-error)
     (define-key haskell-mode-map ghc-next-key        'ghc-goto-next-error)
     (define-key haskell-mode-map ghc-help-key        'ghc-display-errors)
-    (define-key haskell-mode-map ghc-insert-key      'ghc-insert-template)
+    (define-key haskell-mode-map ghc-insert-key      'ghc-insert-template-or-signature)
     (define-key haskell-mode-map ghc-sort-key        'ghc-sort-lines)
-    (define-key haskell-mode-map ghc-check-key       'ghc-save-buffer)
     (define-key haskell-mode-map ghc-toggle-key      'ghc-toggle-check-command)
     (define-key haskell-mode-map ghc-jump-key        'ghc-jump-file)
     (define-key haskell-mode-map ghc-module-key      'ghc-insert-module)
@@ -104,8 +108,16 @@
     (define-key haskell-mode-map ghc-hoogle-key      'haskell-hoogle)
     (define-key haskell-mode-map ghc-shallower-key   'ghc-make-indent-shallower)
     (define-key haskell-mode-map ghc-deeper-key      'ghc-make-indent-deeper)
+    ;(define-key haskell-mode-map ghc-case-split-key  'ghc-case-split)
+    (define-key haskell-mode-map ghc-refine-key      'ghc-refine)
+    (define-key haskell-mode-map ghc-auto-key        'ghc-auto)
+    (define-key haskell-mode-map ghc-prev-hole-key   'ghc-goto-prev-hole)
+    (define-key haskell-mode-map ghc-next-hole-key   'ghc-goto-next-hole)
     (ghc-comp-init)
-    (setq ghc-initialized t))
+    (setq ghc-initialized t)
+    (defadvice save-buffer (after ghc-check-syntax-on-save activate)
+      "Check syntax with GHC when a haskell-mode buffer is saved."
+      (when (eq 'haskell-mode major-mode) (ghc-check-syntax))))
   (ghc-import-module)
   (ghc-check-syntax))
 
@@ -116,13 +128,14 @@
 (defun ghc-debug ()
   (interactive)
   (let ((el-path (locate-file "ghc.el" load-path))
-	(ghc-path (executable-find "ghc"))
+	(ghc-path (executable-find "ghc")) ;; FIXME
 	(ghc-mod-path (executable-find ghc-module-command))
 	(ghc-modi-path (executable-find ghc-interactive-command))
 	(el-ver ghc-version)
 	(ghc-ver (ghc-run-ghc-mod '("--version") "ghc"))
 	(ghc-mod-ver (ghc-run-ghc-mod '("version")))
-	(ghc-modi-ver (ghc-run-ghc-mod '("version") ghc-interactive-command)))
+	(ghc-modi-ver (ghc-run-ghc-mod '("version") ghc-interactive-command))
+	(path (getenv "PATH")))
     (switch-to-buffer (get-buffer-create "**GHC Debug**"))
     (erase-buffer)
     (insert "Path: check if you are using intended programs.\n")
@@ -134,6 +147,14 @@
     (insert (format "\t  ghc.el version %s\n" el-ver))
     (insert (format "\t %s\n" ghc-mod-ver))
     (insert (format "\t%s\n" ghc-modi-ver))
-    (insert (format "\t%s\n" ghc-ver))))
+    (insert (format "\t%s\n" ghc-ver))
+    (insert "\nEnvironment variables:\n")
+    (insert (format "\tPATH=%s\n" path))))
+
+(defun ghc-insert-template-or-signature (&optional flag)
+  (interactive "P")
+  (if flag
+      (ghc-initial-code-from-signature)
+    (ghc-insert-template)))
 
 (provide 'ghc)
